
/*
 * $Id: DLog.cc,v 1.4.2.1 1999/07/29 19:04:39 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "LogMeta.h"
#include "DLog.h"


/* DLog */

DLog::DLog(): 
	theErrCnt(-1), theNonErrCnt(-1),
	thePingRec(0), theSoftRec(0), 
	thePingFilter(0), theSoftFilter(0),
	theRepRateSum(0),
	theReqRate(-1), theReqRateCnt(0), theReqRateState(rrNone) {
}

DLog::~DLog() {
	while (theSources.count()) delete theSources.pop();
}

double DLog::reqRate() const {
	if (thePingRec)
		return thePingRec->reqRate();
	return theReqRateState == rrOK ?
		theReqRateCnt*theReqRate : theReqRate;
}

double DLog::repRate() const {
	if (thePingRec)
		return thePingRec->repRate();
	return theRepRateSum;
}

void DLog::reqRate(double aRate) {
	switch (theReqRateState) {
		case rrNone: // first rate
			theReqRate = aRate;
			theReqRateCnt = 1;
			theReqRateState = rrOK;
			break;

		case rrOK:   // repeat, no conflict before
			if (NetDouble(aRate) == NetDouble(theReqRate)) {
				theReqRateCnt++; // fine, still no conflict
			} else {
				theReqRate = -2; // disable due to conflicts
				theReqRateState = rrConflict;
			}
			break;

		default:     // there were conflicts before
			break;
	}
}

void DLog::repRate(double aRate) {
	theRepRateSum += aRate;
}

void DLog::pingFilter(Filter aFilter) {
	Assert(!thePingFilter);
	thePingFilter = aFilter;
}

void DLog::softFilter(Filter aFilter) {
	Assert(!theSoftFilter);
	theSoftFilter = aFilter;
}

void DLog::noteRec(const StatPingRec &rec) {
	if (thePingFilter) {
		thePingRecAcc += rec;
		if (theCutter.update(theProgress.theDuration))
			pushAccStats();
	}
}

void DLog::noteRec(const StatSoftRec &rec) {
	if (theSoftFilter) {
		theSoftRecAcc += rec;
		if (theCutter.update(theProgress.theDuration))
			pushAccStats();
	}
}

void DLog::pushAccStats() {
	if (thePingFilter) {
		thePingRec = &thePingRecAcc;
		(*thePingFilter)(*this);
		thePingRec = 0;
		thePingRecAcc.restart();
	}

	if (theSoftFilter) {
		theSoftRec = &theSoftRecAcc;
		(*theSoftFilter)(*this);
		theSoftRec = 0;
		theSoftRecAcc.restart();
	}
}
