
/*
 * $Id: StatPhase.cc,v 1.4.2.1 1999/06/15 23:19:26 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iomanip.h>

#include "Clock.h"
#include "StatPhase.h"
#include "StatPhaseMgr.h"
#include "ErrorMgr.h"
#include "Progress.h"
#include "ILog.h"
#include "LogComment.h"
#include "logTags.h"
#include "globals.h"

extern bool ShutdownNow(); // XXX: ARGH!

/* StatPhaseRec */

StatPhaseRec::StatPhaseRec() {
}

void StatPhaseRec::store(OLog &log) const {
	StatPingRec::store(log);
	StatSoftRec::store(log);
	theXactRdSz.store(log);
	theXactTtlVsSz.store(log);
	theGoal.store(log);
	theIdealHR.store(log); // XXX
	theChbR.store(log); // XXX
	theConnUse.store(log); // XXX
	theXactLifeVsClass.store(log); // XXX
	theConnLifeVsUse.store(log); // XXX
	// store error histogram (XXX: as portable as errno)
	if (ErrorMgr::Mgr()) {
		log << ErrorMgr::Mgr()->errorCnts();
	} else {
		Array <int> cnt;
		log << cnt;
	}
}

void StatPhaseRec::load(ILog &log) {
	StatPingRec::load(log);
	StatSoftRec::load(log);
	theXactRdSz.load(log);
	theXactTtlVsSz.load(log);
	theGoal.load(log);
	if (!log.eoe()) {
		theIdealHR.load(log); // XXX
		theChbR.load(log); // XXX
		if (!log.eoe()) {
			theConnUse.load(log); // XXX
			theXactLifeVsClass.load(log); // XXX
			theConnLifeVsUse.load(log); // XXX
			// load error histogram for now
			Array <int> cnts;
			log >> cnts;
			if (ErrorMgr::Mgr())
				ErrorMgr::Mgr()->addErrorCnts(cnts);
		}
	}
}

StatPhaseRec &StatPhaseRec::operator +=(const StatPhaseRec &r) {
	StatPingRec::operator +=(r);
	StatSoftRec::operator +=(r);
	theXactRdSz.add(r.theXactRdSz);
	theXactTtlVsSz.add(r.theXactTtlVsSz);
	theGoal += r.theGoal;
	theConnLifeVsUse.add(r.theConnLifeVsUse);
	return *this;
}

/* StatPhase */

StatPhase::StatPhase(const char *aName, int aLogTag):
	theName(aName), theLogTag(aLogTag), theLock(0), theMgr(0) {
}

void StatPhase::start(StatPhaseMgr *aMgr) {
	Assert(!theMgr);
	theMgr = aMgr;
	StatInt::start();
}

void StatPhase::flush() {
	duration(TheClock - theIntStart);

	OLog &log = TheLog();
	log << bege(theLogTag);
	StatPhaseRec::store(log);
	log << ende;

	report();
}

void StatPhase::checkpoint()  {
	// if nobody prohibits us from stopping and local goal is reached
	if (!theLock && TheProgress >= theGoal) {
		Assert(theMgr);
		theMgr->noteDone(this);
		(void)ShutdownNow(); // early detection
	}
}


void StatPhase::report() const {
	Comment(1)
		<< "p-" << name()
		<< ' ' << setw(6) << TheProgress.theXactComplCnt
		<< ' ' << setw(6) << Ratio(theXactLvl.decCnt(), duration().secd())
		<< ' ' << setw(6) << theRealHR.dhp()
		<< ' ' << setw(3) << theErrCnt
		<< endc;
}
