
/*
 * $Id: ErrorMgr.h,v 1.5.2.1 1999/06/15 23:19:28 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef ERROR_MGR_H
#define ERROR_MGR_H

#include "Array.h"

// source location holder
class SrcLoc {
	public:
		SrcLoc(const char *aFName = 0, int aLineNo = 1);
		ostream &print(ostream &os) const;

		SrcLoc &operator ++() { theLineNo++; return *this; }

	protected:
		const char *theFName;
		int theLineNo;
};

inline ostream &operator <<(ostream &os, const SrcLoc &l) { return l.print(os); }
#define Here SrcLoc(__FILE__, __LINE__)

// central point for all error events
class ErrorMgr {
	public:
		static void Configure(ErrorMgr *aMgr);
		static ErrorMgr *Mgr();

	public:
		ErrorMgr(int aMaxErrNo); // defaults
		virtual ~ErrorMgr() {}

		int maxErrNo() const { return theMaxErrNo; }
		void maxErrNo(int aMaxErrNo);

		int totalCnt() const { return theTotCnt; }

		virtual void noteError(int e);
		virtual void reportError(const SrcLoc &loc, int e);

		virtual bool needExpl(int e);

		const Array<int> &errorCnts() const { return theErrCnts; }
		void addErrorCnts(const Array<int> &cnts);

		virtual const char *explStr(int e) const;

	protected:
		int theVerbLvl;             // how often to explain errors
		int theMaxErrNo;

		Array<int> theErrCnts;      // error counts
		Array<int> theExplDlines;   // time of next report
		int theTotCnt;              // total number of errors
};

extern void NoteError(int e);    // just silently count it
extern bool ExplainError(int e); // count and return true if should explain
                                 // count and maybe print a std explanation
extern void ReportErrorLoc(const SrcLoc &, int e);  
extern void FatalErrorLoc(const SrcLoc &, int e);   // count, report, exit

// macros to auto supply source location
#define ReportError(e) ReportErrorLoc(Here, (e))
#define FatalError(e) FatalErrorLoc(Here, (e))

#endif
