
/*
 * $Id: NotifMsg.h,v 1.1.2.6 1999/06/14 20:59:28 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef PG_NOTIFICATION_MESSAGE_H
#define PG_NOTIFICATION_MESSAGE_H

#include "NetDouble.h"

class NetAddr;

// messages used to send notifications to some [remote] monitor

class NotifMsg {
	public:
		enum Type { mtNone, mtStatus };

	public:
		NotifMsg(int anId = mtNone): theId(anId), theSize(sizeof(*this)) {};

	protected:
		void hton();
		void ntoh();

	public:
		int theId;
		int theSize;
};

class StatusNotifMsg: public NotifMsg {
	public:
		StatusNotifMsg(const String &aLabel = 0);

		void hton();
		void ntoh();

	public:
		char theLabel[16];     // a short 0-terminated description of an experiment
		Time theStartTime;
		Time theSndTime;       // when the message is sent
		Time theRespTime;
		NetDouble theTput;     // xactions / sec
		NetDouble theBwidth;   // network I/O bandwidth (r/w) [B/s]
		NetDouble theDHR;
		int theXactCnt;        // total so far
		int theSockCnt;
};

// status message forwarded by the first recepient
class StatusFwdMsg: public StatusNotifMsg {
	public:
		StatusFwdMsg();
		StatusFwdMsg(const StatusNotifMsg &m, Time aRcvTime, const NetAddr &aSndAddr);

		void hton();
		void ntoh();

	public:
		Time theRcvTime;       // when the message is received
		struct {               // original sender (set by first recepient)
			in_addr addr;
			int port;
		} theSndAddr;
		int theCopyCnt;        // number of copies forwarded (to various recepients)
};

#endif
