
/*
 * $Id: BigSize.cc,v 1.2 1999/05/24 00:52:30 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <limits.h>

#include "Assert.h"
#include "BigSize.h"


/* static creators */

BigSize BigSize::MB(int n) {
	const int factor = INT_MAX/(1024*1024); // MBs in a chunk
	Assert(factor > 0);
	BigSize sz;
	sz.theCnt = n / factor; // chunks in n
	sz.theAcc = (n % factor) * (1024*1024); // leftovers
	return sz;
}

/* dynamic methods */

BigSize::BigSize(): theAcc(0), theCnt(0) {
}

BigSize &BigSize::operator +=(const BigSize &bsz) {
	theCnt += bsz.theCnt;
	(*this) += bsz.theAcc;
	return *this;
}

BigSize &BigSize::operator +=(int sz) {
	const int space = INT_MAX - theAcc;
	if (space <= sz) { // overflow
		theAcc = sz - space;
		theCnt++;
	} else {
		theAcc += sz;
	}
	return *this;
}

double BigSize::operator /=(const BigSize &sz) const {
	return byted()/sz.byted();
}

double BigSize::byted() const {
	return theCnt*(double)INT_MAX + theAcc;
}

