
/*
 * $Id: Rnd.cc,v 1.5 1999/05/26 19:03:25 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <iostream.h>

#include "Assert.h"
#include "Rnd.h"

// RND_MAX is MAX_INT if sizeof(int) is 4
#define RND_MAX 0x7fffffff

RndGen::State RndGen::TheDefSeed = 1;

// a negative seed will be set (to the default value) before first use
// see trial();
RndGen::RndGen(State aSeed): theSeed(aSeed), theState(aSeed) {
	if (aSeed >= 0)
		seed(aSeed);
}

void RndGen::seed(State aSeed) {
	Assert(aSeed >= 0);
	theSeed = aSeed;
	theState = aSeed ? aSeed : (RND_MAX/2 | 1);
}

// uniform on [0, 2^31)
RndGen::State RndGen::ltrial() {
	// set initial seed to default if still undefined
	if (theSeed < 0)
		seed(TheDefSeed);

	// one can never get out of zero state
	Assert(theState);

	/*
	 * Compute x[n + 1] = (7^5 * x[n]) mod (2^31 - 1).
	 * From "Random number generators: good ones are hard to find",
	 * Park and Miller, Communications of the ACM, vol. 31, no. 10,
	 * October 1988, p. 1195.
	 *
	 * note: (2^31 - 1) == 127773 * (7^5) + 2836
	 */
	const int h = theState;
	const int hi = h / 127773;
	const int lo = h % 127773;
	const int x = 16807 * lo - 2836 * hi;

	theState = x ? x : RND_MAX;
	return x < 0 ? x + RND_MAX : x;
}


/* RndDistr */

ostream &RndDistr::print(ostream &os) const {
	return os << pdfName() << '(' << mean() << ')';
}
