
/*
 * $Id: FileScanner.h,v 1.1.2.1 1999/06/13 04:52:52 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef XSTD_FILE_SCANNER_H
#define XSTD_FILE_SCANNER_H

#include <sys/types.h>
#include <unistd.h>

#include "Array.h"
#include "Time.h"

// file scanner is a common interface to select(2) and poll(2)

typedef enum { dirNone = -1, dirRead = 0, dirWrite = 1 } IODir;

class FileScanUser {
	public:
		virtual ~FileScanUser();

		// default implementation will just abort()
		virtual void noteReadReady(int /*fd*/);
		virtual void noteWriteReady(int /*fd*/);
};

// reservation receipt
class FileScanReserv {
	public:
		FileScanReserv();
		FileScanReserv(int aFD, IODir aDir);

		operator bool() const { return reserved(); }
		bool reserved() const { return theFD >= 0; }
		bool ready() const { return isReady; }
		int fd() const { return theFD; }
		IODir dir() const { return theDir; }

		void ready(bool be) { isReady = be; }
		void clear() { theFD = -1; }

	protected:
		int theFD;
		IODir theDir;
		bool isReady;
};

// Ticker is notified about various stages of a scan process
class FileScanTicker {
	public:
		virtual ~FileScanTicker() {}

		virtual void begCycle(int readyCount) = 0;
		virtual void endCycle() = 0;
		virtual void tick() = 0;
};

// the "scanner" itself
class FileScanner {
	public:
		typedef FileScanUser User;
		typedef FileScanTicker Ticker;

	public:
		FileScanner();
		virtual ~FileScanner();

		// scanner's name (e.g., system call for syscall wrappers)
		virtual const char *name() const = 0;

		virtual void configure(int fdLimit);
		void ticker(Ticker *aTicker);

		bool idle() const { return theResCount <= 0; }

		virtual FileScanReserv setFD(int fd, IODir dir, User *p);
		virtual void clearFD(int fd, IODir dir);
		void clearRes(FileScanReserv &res) { clearFD(res.fd(), res.dir()); res.clear(); }

		// returns a positive number if some files where ready; -1 on error
		// calls applicable User methods for ready files
		// null timeout means wait until error or at least one file is ready
		int scan(Time *timeout = 0);

	protected:
		// returns "hot count" (see scan code)
		virtual int sweep(Time *timeout = 0) = 0;

		// must set fd if the indexed user is ready
		virtual User *readyUser(int idx, IODir dir, int &fd) = 0;

	protected:
		Ticker *theTicker;
		int theResCount;
};

// we must add an iterator to make the interface complete

#endif
